#include"sim.h"
#include"ihv_lex.h"
#include"db.h"
#include<signal.h>
#include"stats.h"
#include"interactive.h"
#include<getopt.h>

char* default_script =
    "globals\n"
    "run\n"
    "export to \"tree.ts\"\n";

void quitproc(int sig)
{
    fprintf(stderr, "Interrupt Received.  Exiting...\n");
    process_end();
}

int main(int argc, char** argv)
{
    int i;
    flog = stdout;
    config_t *config = config_load(argc, argv);

    signal(SIGINT, quitproc);
    signal(SIGTERM, quitproc);

    initialize_globals(config);
//    print_params(flog);

    if(config->numfiles > 0)
    {
        fprintf(flog, "Processing %d files...\n", config->numfiles);

        if(config->interactive)
            command_line();
/*
        else if(script)
        {
            yyparse();  // need to feed script file into yyparse
            //command_line();
        }
*/
        else
        {
            YY_BUFFER_STATE buffer =
                (YY_BUFFER_STATE) yy_scan_string(default_script);
            yy_switch_to_buffer(buffer);
            yyparse();
            yy_delete_buffer(buffer);
        }
    }
    else
    {
/*
        if(script)
            yyparse();
*/

        process_live();
    }

    if(fgraph) { print_graph(fgraph); fclose(fgraph); }
    print_stats(flog);
    print_stats_summary(flog);
    fclose(flog);
    free_globals();
    config_free(config);
    return 0;
}
