// sim.h
#ifndef SIM_H
#define SIM_H

#include<glib.h>
#include<stdio.h>
#include<stdlib.h>
#include<assert.h>
#include<unistd.h>
#include"constants.h"
#include"protocol.h"
//#include"hashtable.h"
//#include"hashtable_itr.h"
#include"adhic.h"
#include"config.h"

/* Data Structures */
typedef struct count
{
    int count[MAX_TIME];
} count_t;

typedef struct vector
{
    int* val;
    int max_size;
    int size;
} vector_t;

typedef struct sig_stack_elem
{
    png_op_t*  pngs;
    int        match;
} sig_stack_elem_t;

typedef struct sig_stack
{
    sig_stack_elem_t stack[MAX_DEPTH];
    int size;
} sig_stack_t;

/* Node Operations */
node_t* node_head();
void    head_set(node_t* node);
node_t* node_default();
node_t* node_png_create_with_png_op_id(png_op_t* pnglist, node_t* parent, int val); // These names are getting sad and the api is going to hell.
node_t* node_png_create_with_id(png_t* png, node_t* parent, int val);
node_t* node_png_create(png_t* png, node_t* parent);
void    node_print(FILE* file, node_t* node);
void    node_print_summary(FILE* file, node_t* node);
void    node_print_packets(FILE* file, node_t* node);
void    node_print_flows(FILE* file, node_t* node);

void    node_update_child(node_t* parent, node_t* old_child, node_t* new_child);
node_t* node_queue_create(node_t* parent);
node_t* node_default_find();
node_t* node_find(node_t* node, int id);
void    node_queue_print_flows(FILE* file, node_t* node);
boolean node_png_contains(node_t* node, png_t* png);
void    visit_queues(node_t* node, void (*func)(node_t* node, void* data), void* data);

/* PNG Ops */

png_t*  png_create();
png_t*  png_init(png_t* png, int offset, u_char* vals);
void    png_op_add (png_op_t* pngs, png_t* png);
void    png_free(png_t* png);
void    png_print(FILE* file, png_t* png);
void    pngs_print(FILE* file, png_op_t* pngs);
png_t*  png_from_vector(int n, struct vector* vector);
void    png_op_copy(png_op_t* dst, png_op_t* src);
png_op_t* png_op_create();
//hashtable_t* packet2pngs(hashtable_t* pngs, qcap_packet_t* packet);

//png_t* png_find_optimal(hashtable_t* pngs, int size);
//png_t* png_find_sufficient(hashtable_t* pngs, int size);
boolean png_equals(png_t* one, png_t* two);


// GLIB specific replacements
GHashTable* GLIB_packet2pngs(GHashTable* pngs, const u_char* packet,
                             unsigned int len); // To replace packet2pngs
png_t* GLIB_png_find_optimal(GHashTable* pngs, int size);
png_t* GLIB_png_find_sufficient(GHashTable* pngs, int size);
void GLIB_destroy_pngs(GHashTable* pngs);

/* Misc */

void    initialize_globals(const config_t *config);
void    print_stats(FILE* file);
void    print_stats_summary(FILE* file);
void    iterate();
void    process_live();
void    process_end();
void    packet_show(FILE* file, const u_char* packet, unsigned int len,
                    int start, int length);
unsigned int hashfn(void* val);
guint   png_hash_fn(gconstpointer key);     // Just calls hashfn but needed for glib
boolean hasheqn(void* val1, void* val2);
gint    png_hash_cmp(gconstpointer a, gconstpointer b); // Just calls hasheqn but needed for glib
void hash_png_free(void* png);
int     mature_packet_count();
void print_with_prefix(FILE* file, char* prefix, char* buffer);

/* Utils */

char*   copystr(char* str);
vector_t* vector_new(int n);
void    vector_add(vector_t* vector, int n);
//int     hashtable_sum(hashtable_t* table);
inline float percent(int x, int y);
/* Variables we export */

extern const config_t *config;
extern int num_queues;              // The current number of queues
extern int N;                       // Size of all pngs
extern int noheaders;               // Should we look in headers?
extern int initial_queues;          // 
extern int initial_pngs;            // From the old algorithm
extern char* queue_file;
extern int simtime;                 // Current time period (# of ticks)
extern int tick_packet_count;
extern int datafile;                // Index of the datafile being read
extern FILE* flog;                  // Log file
extern FILE* fgraph;                // Graph log
extern boolean old_algorithm;       // Should we use the old algorithm?

extern boolean globals;             // Print globals every time period?
extern int total_packets;           // Should this be a long long?
extern int ip_offset;               // IP header's position within packets

#endif
