class CaptureRequest < ActiveRecord::Base
  # Request a packet capture
  # @param tree The tree on which to do the capture
  # @param node The node's unique id in the tree
  # @return Returns the new capture request from the database
  def self.start(tree, node)
    # make sure it's not a duplicate first
    return nil if find(:first,
                       :conditions => "tree_id=#{tree.id} AND node_id=#{node}")
    r = self.new(:tree_id => tree.id, :node_id => node)
    r.save
    r
  end

  # End a packet capture
  # @param request The ID of the packet capture request to stop
  def self.end(request)
    delete(request)
  end

  # Gets a list of all on-going packet capture requests
  # @return Retuns an array of CaptureRequest objects, which are the on-going
  #         requests for packet captures from the database
  def self.current(tree)
    find(:all, :conditions => "tree_id=#{tree.id}")
  end

  # Gets the unique node id for a packet capture request
  # @param request The ID of an on-going packet capture request
  # @return Returns the unique node id which this capture request is for
  def self.get_node(request)
    r = find(request)
    if r then r.node_id.to_i else nil end
  end
end
