require 'svg'
require 'xml/libxml'

# A mix-in for the InternalNode and TerminalNode classes
module Node
  # The node's left child (An InternalNode or TerminalNode)
  attr :left, true
  # The node's right child (An InternalNode or TerminalNode)
  attr :right, true
  # The node's parent (An InternalNode or TerminalNode)
  attr :parent, true
  # The amount of zoom to use when displaying the node (for highlighting
  # traffic in the tree view)
  attr :zoom, true

  # Is the node a left child of its parent?
  def leftchild?
    return (parent && parent.left == self)
  end
  # Is the node a right child of its parent?
  def rightchild?
    return (parent && parent.right == self)
  end
  
  # SVG representation of the node (from SVG module). This is not the final
  # XML representation
  attr :svg, true
  # SVG representation of the line from this node to its left child
  attr :svgleft, true
  # SVG representation of the line from this node to its right child
  attr :svgright, true
end
