# NodePngrams associate a p,n-gram in the database with an InternalNode
class NodePngram < ActiveRecord::Base
  # Find all the p,n-grams for a node
  # @param n An InternalNode object
  # @return Returns an array of read-only NodePngram objects which are the
  #         p,n-grams for the given node.  The returned objects include the
  #         "p" and "n" fields from the Pngram class.
  def self.find_for_node(n)
    png_hash_init(n.revision_id) if !(@png_hash && @png_hash[n.revision_id])
    @png_hash[n.revision_id][n.id]
  end

protected
  def self.png_hash_init(revid)
    h = ((@png_hash ||= {})[revid] = {})

    find(:all,
         :joins => "INNER JOIN internal_nodes ON" +
                   " internal_node_id = internal_nodes.id" +
                   " INNER JOIN pngrams ON pngram_id = pngrams.id",
         :conditions => "revision_id=#{revid}").each { |i|
      h[i.internal_node_id] ||= []
      h[i.internal_node_id] << i
    }
  end
end
